@@ -1,5 +1,6 @@
 // Copyright (c)2004 by Edward Counce, All rights reserved. 
 // Copyright (c)2006-7 by Salvador E. Tropea, All rights reserved.
+// Copyright (c)2008 by Daniel Beer, All rights reserved.
 // This file is part of drawtiming.
 //
 // Drawtiming is free software; you can redistribute it and/or modify
@@ -18,6 +19,7 @@
 
 #include "timing.h"
 #include <map>
+#include <fstream>
 using namespace std;
 using namespace timing;
 using namespace Magick;
@@ -279,122 +281,10 @@ ostream &operator<< (ostream &f, const depdata &dep) {
   return f;
 }
 
-// ------------------------------------------------------------
-
-diagram::diagram (void) {
-  width = height = 0;
-}
-
-diagram::diagram (const diagram &d) {
-  *this = d;
-}
-
-// ------------------------------------------------------------
-
-diagram &diagram::operator= (const diagram &d) {
-  width = d.width;
-  height = d.height;
-  return *this;
-}
-
-// ------------------------------------------------------------
-
-void diagram::render (const data &d, double scale) {
-  int base_width, base_height;
-  base_size (d, base_width, base_height);
-
-  width = (int)(scale * base_width);
-  height = (int)(scale * base_height);
-
-  render_common (d, scale, scale);
-}
-
-// ------------------------------------------------------------
-
-void diagram::render (const data &d, int w, int h, bool fixAspect) {
-  int base_width, base_height;
-  base_size (d, base_width, base_height);
-
-  width = w;
-  height = h;
-
-  double hscale = w / (double)base_width;
-  double vscale = h / (double)base_height;
-
-  if (fixAspect) {
-      // to maintain aspect ratio, and fit the image:
-      hscale = vscale = min (hscale, vscale);
-  }
-
-  render_common (d, hscale, vscale);
-}
-
-// ------------------------------------------------------------
-
-void diagram::render_common (const data &d, double hscale, double vscale) {
-
-  push_back (DrawablePushGraphicContext ());
-  push_back (DrawableScaling (hscale, vscale));
-  push_back (DrawableFont (vFont, AnyStyle, 100, AnyStretch));
-  push_back (DrawablePointSize (vFontPointsize));
-  push_back (DrawableStrokeWidth(vLineWidth));
-  push_back (DrawableStrokeColor ("black"));
-
-  int labelWidth = label_width (d);
-
-  // draw a "scope-like" diagram for each signal
-  map<signame,int> ypos;
-  int y = 0;
-  for (signal_sequence::const_iterator i = d.sequence.begin ();
-       i != d.sequence.end (); ++ i) {
-    const sigdata &sig = d.find_signal (*i);
-    push_text (vCellWrm, y + vCellHtxt, *i);
-    ypos[*i] = y;
-    int x = labelWidth + vCellWtsep;
-    sigvalue last;
-    for (value_sequence::const_iterator j = sig.data.begin ();
-	 j != sig.data.end (); ++ j) {
-      draw_transition (x, y, last, *j);
-      last = *j;
-      x += vCellW;
-    }
-    y += vCellHt + vCellHdel * sig.maxdelays;
-  }
-
-  // draw the smooth arrows indicating the triggers for signal changes
-  for (list<depdata>::const_iterator i = d.dependencies.begin (); 
-       i != d.dependencies.end (); ++ i)
-    draw_dependency (labelWidth + vCellWtsep + vCellWrm + vCellW * i->n_trigger,
-                     vCellHt/2 + ypos[i->trigger],
-		     labelWidth + vCellWtsep + vCellWrm + vCellW * i->n_effect,
-                     vCellHt/2 + ypos[i->effect]);
-
-  // draw the timing delay annotations
-  for (list<delaydata>::const_iterator i = d.delays.begin (); 
-       i != d.delays.end (); ++ i)
-    draw_delay (labelWidth + vCellWtsep + vCellWrm + vCellW * i->n_trigger,
-                vCellHt/2 + ypos[i->trigger],
-		labelWidth + vCellWtsep + vCellWrm + vCellW * i->n_effect,
-                vCellHt/2 + ypos[i->effect],
-		ypos[i->trigger] + vCellHt + vCellHdel * i->offset + vCellHtdel,
-                i->text);
-
-  push_back (DrawablePopGraphicContext ());
-}
-
-// ------------------------------------------------------------
-// add text to the diagram
-
-void diagram::push_text (double xpos, double ypos, const string &text) {
-  push_back (DrawableStrokeWidth(1));
-  push_back (DrawableText (xpos, ypos, text));
-  push_back (DrawableStrokeWidth(vLineWidth));
-}
-
 // ------------------------------------------------------------
 // calculate the required label width
 
-int diagram::label_width (const data &d) const {
+static int label_width (const data &d) {
   int labelWidth = 0;
   Image img;
   TypeMetric m;
@@ -413,7 +303,7 @@ int diagram::label_width (const data &d) const {
 // ------------------------------------------------------------
 // calculate the basic height and width required before scaling
 
-void diagram::base_size (const data &d, int &w, int &h) const {
+static void base_size (const data &d, int &w, int &h) {
 
   vCellHsep = vCellHt / 8;
   vCellH=vCellHt-vCellHsep;
@@ -434,30 +324,39 @@ void diagram::base_size (const data &d, int &w, int &h) const {
 }
 
 // ------------------------------------------------------------
+// add text to the diagram
 
-void diagram::draw_transition (int x, int y, const sigvalue &last,
-			       const sigvalue &value) {
+static void push_text (gc &gc, double xpos, double ypos, const string &text) {
+  gc.stroke_width (1);
+  gc.text (int (xpos), int (ypos), text);
+  gc.stroke_width (vLineWidth);
+}
+
+// ------------------------------------------------------------
+
+static void draw_transition (gc &gc, int x, int y, const sigvalue &last,
+			     const sigvalue &value) {
 
   switch (value.type) {
   case ZERO:
     switch (last.type) {
     default:
-      push_back (DrawableLine (x, y + vCellH, x + vCellW, y + vCellH));
+      gc.line (x, y + vCellH, x + vCellW, y + vCellH);
       break;
 
     case ONE:
-      push_back (DrawableLine (x, y + vCellHsep, x + vCellW/4, y + vCellH));
-      push_back (DrawableLine (x + vCellW/4, y + vCellH, x + vCellW, y + vCellH));
+      gc.line (x, y + vCellHsep, x + vCellW/4, y + vCellH);
+      gc.line (x + vCellW/4, y + vCellH, x + vCellW, y + vCellH);
       break;
     
     case Z:
-      push_back (DrawableLine (x, y + vCellHt/2, x + vCellW/4, y + vCellH));
-      push_back (DrawableLine (x + vCellW/4, y + vCellH, x + vCellW, y + vCellH));
+      gc.line (x, y + vCellHt/2, x + vCellW/4, y + vCellH);
+      gc.line (x + vCellW/4, y + vCellH, x + vCellW, y + vCellH);
       break;
 
     case STATE:
-      push_back (DrawableLine (x, y + vCellHsep, x + vCellW/4, y + vCellH));
-      push_back (DrawableLine (x, y + vCellH, x + vCellW, y + vCellH));
+      gc.line (x, y + vCellHsep, x + vCellW/4, y + vCellH);
+      gc.line (x, y + vCellH, x + vCellW, y + vCellH);
       break;
     }
     break;
@@ -465,24 +364,24 @@ void diagram::draw_transition (int x, int y, const sigvalue &last,
   case ONE:
     switch (last.type) {
     default:
-      push_back (DrawableLine (x, y + vCellHsep, x + vCellW, y + vCellHsep));
+      gc.line (x, y + vCellHsep, x + vCellW, y + vCellHsep);
       break;
 
     case ZERO:
     case TICK:
     case PULSE:
-      push_back (DrawableLine (x, y + vCellH, x + vCellW/4, y + vCellHsep));
-      push_back (DrawableLine (x + vCellW/4, y + vCellHsep, x + vCellW, y + vCellHsep));
+      gc.line (x, y + vCellH, x + vCellW/4, y + vCellHsep);
+      gc.line (x + vCellW/4, y + vCellHsep, x + vCellW, y + vCellHsep);
       break;
 
     case Z:
-      push_back (DrawableLine (x, y + vCellHt/2, x + vCellW/4, y + vCellHsep));
-      push_back (DrawableLine (x + vCellW/4, y + vCellHsep, x + vCellW, y + vCellHsep));
+      gc.line (x, y + vCellHt/2, x + vCellW/4, y + vCellHsep);
+      gc.line (x + vCellW/4, y + vCellHsep, x + vCellW, y + vCellHsep);
       break;
 
     case STATE:
-      push_back (DrawableLine (x, y + vCellH, x + vCellW/4, y + vCellHsep));
-      push_back (DrawableLine (x, y + vCellHsep, x + vCellW, y + vCellHsep));
+      gc.line (x, y + vCellH, x + vCellW/4, y + vCellHsep);
+      gc.line (x, y + vCellHsep, x + vCellW, y + vCellHsep);
       break;
     }
     break;
@@ -491,31 +390,31 @@ void diagram::draw_transition (int x, int y, const sigvalue &last,
   case PULSE:
     switch (last.type) {
     default:
-      push_back (DrawableLine (x, y + vCellH, x + vCellW/4, y + vCellHsep));
-      push_back (DrawableLine (x + vCellW/4, y + vCellHsep, x + vCellW/2, y + vCellHsep));
-      push_back (DrawableLine (x + vCellW/2, y + vCellHsep, x + vCellW*3/4, y + vCellH));
-      push_back (DrawableLine (x + vCellW*3/4, y + vCellH, x + vCellW, y + vCellH));
+      gc.line (x, y + vCellH, x + vCellW/4, y + vCellHsep);
+      gc.line (x + vCellW/4, y + vCellHsep, x + vCellW/2, y + vCellHsep);
+      gc.line (x + vCellW/2, y + vCellHsep, x + vCellW*3/4, y + vCellH);
+      gc.line (x + vCellW*3/4, y + vCellH, x + vCellW, y + vCellH);
       break;
 
     case ONE:
     case X:
-      push_back (DrawableLine (x, y + vCellHsep, x + vCellW/2, y + vCellHsep));
-      push_back (DrawableLine (x + vCellW/2, y + vCellHsep, x + vCellW*3/4, y + vCellH));
-      push_back (DrawableLine (x + vCellW*3/4, y + vCellH, x + vCellW, y + vCellH));
+      gc.line (x, y + vCellHsep, x + vCellW/2, y + vCellHsep);
+      gc.line (x + vCellW/2, y + vCellHsep, x + vCellW*3/4, y + vCellH);
+      gc.line (x + vCellW*3/4, y + vCellH, x + vCellW, y + vCellH);
       break;
 
     case Z:
-      push_back (DrawableLine (x, y + vCellHt/2, x + vCellW/4, y + vCellHsep));
-      push_back (DrawableLine (x + vCellW/4, y + vCellHsep, x + vCellW/2, y + vCellHsep));
-      push_back (DrawableLine (x + vCellW/2, y + vCellHsep, x + vCellW*3/4, y + vCellH));
-      push_back (DrawableLine (x + vCellW*3/4, y + vCellH, x + vCellW, y + vCellH));
+      gc.line (x, y + vCellHt/2, x + vCellW/4, y + vCellHsep);
+      gc.line (x + vCellW/4, y + vCellHsep, x + vCellW/2, y + vCellHsep);
+      gc.line (x + vCellW/2, y + vCellHsep, x + vCellW*3/4, y + vCellH);
+      gc.line (x + vCellW*3/4, y + vCellH, x + vCellW, y + vCellH);
       break;
 
     case STATE:
-      push_back (DrawableLine (x, y + vCellH, x + vCellW/4, y + vCellHsep));
-      push_back (DrawableLine (x, y + vCellHsep, x + vCellW/2, y + vCellHsep));
-      push_back (DrawableLine (x + vCellW/2, y + vCellHsep, x + vCellW*3/4, y + vCellH));
-      push_back (DrawableLine (x + vCellW*3/4, y + vCellH, x + vCellW, y + vCellH));
+      gc.line (x, y + vCellH, x + vCellW/4, y + vCellHsep);
+      gc.line (x, y + vCellHsep, x + vCellW/2, y + vCellHsep);
+      gc.line (x + vCellW/2, y + vCellHsep, x + vCellW*3/4, y + vCellH);
+      gc.line (x + vCellW*3/4, y + vCellH, x + vCellW, y + vCellH);
       break;
     }
     break;
@@ -523,35 +422,35 @@ void diagram::draw_transition (int x, int y, const sigvalue &last,
   case UNDEF:
   case X:
     for (int i = 0; i < 4; ++ i) {
-      push_back (DrawableLine (x+i*(vCellW/4), y + vCellH,
-                               x+(i+1)*(vCellW/4), y + vCellHsep));
-      push_back (DrawableLine (x+i*(vCellW/4), y + vCellHsep,
-                               x+(i+1)*(vCellW/4), y + vCellH));
+      gc.line (x+i*(vCellW/4), y + vCellH,
+	       x+(i+1)*(vCellW/4), y + vCellHsep);
+      gc.line (x+i*(vCellW/4), y + vCellHsep,
+	       x+(i+1)*(vCellW/4), y + vCellH);
     }
     break;
   
   case Z:
     switch (last.type) {
     default:
-      push_back (DrawableLine (x, y + vCellHt/2, x + vCellW, y + vCellHt/2));
+      gc.line (x, y + vCellHt/2, x + vCellW, y + vCellHt/2);
       break;
 
     case ZERO:
     case TICK:
     case PULSE:
-      push_back (DrawableLine (x, y + vCellH, x + vCellW/4, y + vCellHt/2));
-      push_back (DrawableLine (x + vCellW/4, y + vCellHt/2, x + vCellW, y + vCellHt/2));
+      gc.line (x, y + vCellH, x + vCellW/4, y + vCellHt/2);
+      gc.line (x + vCellW/4, y + vCellHt/2, x + vCellW, y + vCellHt/2);
       break;
 
     case ONE:
-      push_back (DrawableLine (x, y + vCellHsep, x + vCellW/4, y + vCellHt/2));
-      push_back (DrawableLine (x + vCellW/4, y + vCellHt/2, x + vCellW, y + vCellHt/2));
+      gc.line (x, y + vCellHsep, x + vCellW/4, y + vCellHt/2);
+      gc.line (x + vCellW/4, y + vCellHt/2, x + vCellW, y + vCellHt/2);
       break;
 
     case STATE:
-      push_back (DrawableLine (x, y + vCellHsep, x + vCellW/8, y + vCellHt/2));
-      push_back (DrawableLine (x, y + vCellH, x + vCellW/8, y + vCellHt/2));
-      push_back (DrawableLine (x + vCellW/8, y + vCellHt/2, x + vCellW, y + vCellHt/2));
+      gc.line (x, y + vCellHsep, x + vCellW/8, y + vCellHt/2);
+      gc.line (x, y + vCellH, x + vCellW/8, y + vCellHt/2);
+      gc.line (x + vCellW/8, y + vCellHt/2, x + vCellW, y + vCellHt/2);
       break;
     }
     break;
@@ -560,40 +459,40 @@ void diagram::draw_transition (int x, int y, const sigvalue &last,
     switch (last.type) {
     default:
       if (value.text != last.text) {
-	push_back (DrawableLine (x, y + vCellHsep, x + vCellW/4, y + vCellH));
-	push_back (DrawableLine (x, y + vCellH, x + vCellW/4, y + vCellHsep));
-	push_back (DrawableLine (x + vCellW/4, y + vCellHsep, x + vCellW, y + vCellHsep));
-	push_back (DrawableLine (x + vCellW/4, y + vCellH, x + vCellW, y + vCellH));
-	push_text (x + vCellW/4, y + vCellHtxt, value.text);
+	gc.line (x, y + vCellHsep, x + vCellW/4, y + vCellH);
+	gc.line (x, y + vCellH, x + vCellW/4, y + vCellHsep);
+	gc.line (x + vCellW/4, y + vCellHsep, x + vCellW, y + vCellHsep);
+	gc.line (x + vCellW/4, y + vCellH, x + vCellW, y + vCellH);
+	push_text (gc, x + vCellW/4, y + vCellHtxt, value.text);
       }
       else {
-	push_back (DrawableLine (x, y + vCellHsep, x + vCellW, y + vCellHsep));
-	push_back (DrawableLine (x, y + vCellH, x + vCellW, y + vCellH));
+	gc.line (x, y + vCellHsep, x + vCellW, y + vCellHsep);
+	gc.line (x, y + vCellH, x + vCellW, y + vCellH);
       }
       break;
 
     case ZERO:
     case TICK:
     case PULSE:
-      push_back (DrawableLine (x, y + vCellH, x + vCellW/4, y + vCellHsep));
-      push_back (DrawableLine (x + vCellW/4, y + vCellHsep, x + vCellW, y + vCellHsep));
-      push_back (DrawableLine (x, y + vCellH, x + vCellW, y + vCellH));
-      push_text (x + vCellW/4, y + vCellHtxt, value.text);
+      gc.line (x, y + vCellH, x + vCellW/4, y + vCellHsep);
+      gc.line (x + vCellW/4, y + vCellHsep, x + vCellW, y + vCellHsep);
+      gc.line (x, y + vCellH, x + vCellW, y + vCellH);
+      push_text (gc, x + vCellW/4, y + vCellHtxt, value.text);
       break;
     
     case ONE:
-      push_back (DrawableLine (x, y + vCellHsep, x + vCellW/4, y + vCellH));
-      push_back (DrawableLine (x + vCellW/4, y + vCellH, x + vCellW, y + vCellH));
-      push_back (DrawableLine (x, y + vCellHsep, x + vCellW, y + vCellHsep));
-      push_text (x + vCellW/4, y + vCellHtxt, value.text);
+      gc.line (x, y + vCellHsep, x + vCellW/4, y + vCellH);
+      gc.line (x + vCellW/4, y + vCellH, x + vCellW, y + vCellH);
+      gc.line (x, y + vCellHsep, x + vCellW, y + vCellHsep);
+      push_text (gc, x + vCellW/4, y + vCellHtxt, value.text);
       break;
     
     case Z:
-      push_back (DrawableLine (x, y + vCellW/4, x + vCellW/8, y + vCellH));
-      push_back (DrawableLine (x, y + vCellW/4, x + vCellW/8, y + vCellHsep));
-      push_back (DrawableLine (x + vCellW/8, y + vCellH, x + vCellW, y + vCellH));
-      push_back (DrawableLine (x + vCellW/8, y + vCellHsep, x + vCellW, y + vCellHsep));
-      push_text (x + vCellW/8, y + vCellHtxt, value.text);
+      gc.line (x, y + vCellW/4, x + vCellW/8, y + vCellH);
+      gc.line (x, y + vCellW/4, x + vCellW/8, y + vCellHsep);
+      gc.line (x + vCellW/8, y + vCellH, x + vCellW, y + vCellH);
+      gc.line (x + vCellW/8, y + vCellHsep, x + vCellW, y + vCellHsep);
+      push_text (gc, x + vCellW/8, y + vCellHtxt, value.text);
       break;
     }
   }
@@ -601,80 +500,447 @@ void diagram::draw_transition (int x, int y, const sigvalue &last,
 
 // ------------------------------------------------------------
 
-void diagram::draw_dependency (int x0, int y0, int x1, int y1) {
+static void draw_dependency (gc &gc, int x0, int y0, int x1, int y1) {
   list<Coordinate> shaft, head;
 
-  push_back (DrawablePushGraphicContext ());
-  push_back (DrawableStrokeColor ("blue"));
+  gc.push ();
+  gc.stroke_color ("blue");
 
   if (x0 == x1) {
     int w = vCellW/20, h = vCellHt/6, h2 = vCellHt/10;
 
     if (y0 < y1) {
       y1 -= vCellHt/4;
-      push_back (DrawableLine (x0, y0, x1, y1));
-      push_back (DrawableFillColor ("blue"));
+      gc.line (x0, y0, x1, y1);
+      gc.fill_color ("blue");
       head.push_back (Coordinate (x1, y1));
       head.push_back (Coordinate (x1 - w, y1 - h));
       head.push_back (Coordinate (x1, y1 - h2));
       head.push_back (Coordinate (x1 + w, y1 - h));
-      push_back (DrawablePolygon (head));
+      gc.polygon (head);
     }
     else {
       y1 += vCellHt/4;
-      push_back (DrawableLine (x0, y0, x1, y1));
-      push_back (DrawableFillColor ("blue"));
+      gc.line (x0, y0, x1, y1);
+      gc.fill_color ("blue");
       head.push_back (Coordinate (x1, y1));
       head.push_back (Coordinate (x1 - w, y1 + h));
       head.push_back (Coordinate (x1, y1 + h2));
       head.push_back (Coordinate (x1 + w, y1 + h));
-      push_back (DrawablePolygon (head));
+      gc.polygon (head);
     }
   }
   else {
     int h = vCellHt/10, w1 = vCellW/12, w2 = vCellW/20;
     x1 -= vCellW/16;
-    push_back (DrawableFillColor ("none"));
-    push_back (DrawableFillOpacity (0));
+    gc.fill_color ("none");
+    gc.fill_opacity (0);
     shaft.push_back (Coordinate (x0, y0));
     shaft.push_back (Coordinate ((x0 + x1) / 2, y1));
     shaft.push_back (Coordinate ((x0 + x1) / 2, y1));
     shaft.push_back (Coordinate (x1, y1));
-    push_back (DrawableBezier (shaft));
-    push_back (DrawableFillColor ("blue"));
+    gc.bezier (shaft);
+    gc.fill_color ("blue");
     head.push_back (Coordinate (x1, y1));
     head.push_back (Coordinate (x1 - w1, y1 - h));
     head.push_back (Coordinate (x1 - w2, y1));
     head.push_back (Coordinate (x1 - w1, y1 + h));
-    push_back (DrawablePolygon (head));
+    gc.polygon (head);
   }
 
-  push_back (DrawablePopGraphicContext ());
+  gc.pop ();
 }
 
 // ------------------------------------------------------------
 
-void diagram::draw_delay (int x0, int y0, int x1, int y1, int y2, 
-			  const string &text) {
+static void draw_delay (gc &gc, int x0, int y0, int x1, int y1, int y2,
+			const string &text) {
   list<Coordinate> head;
 
-  push_back (DrawablePushGraphicContext ());
-  push_back (DrawableStrokeColor ("blue"));
+  gc.push ();
+  gc.stroke_color ("blue");
 
   if (x0 == x1) 
-    push_back (DrawableLine (x0, y0, x1, y1));
+    gc.line (x0, y0, x1, y1);
   else {
-    push_back (DrawableText (x0 + vCellWtsep, y2 - vCellHt/16, text));
-    push_back (DrawableLine (x0, y0, x0, y2 + vCellHt/8));
-    push_back (DrawableLine (x1, y1, x1, y2 - vCellHt/8));
-    push_back (DrawableLine (x0, y2, x1, y2));
-    push_back (DrawableFillColor ("blue"));
+    gc.text (x0 + vCellWtsep, y2 - vCellHt/16, text);
+    gc.line (x0, y0, x0, y2 + vCellHt/8);
+    gc.line (x1, y1, x1, y2 - vCellHt/8);
+    gc.line (x0, y2, x1, y2);
+    gc.fill_color ("blue");
     head.push_back (Coordinate (x1, y2));
     head.push_back (Coordinate (x1 - vCellW/12, y2 - vCellHt/10));
     head.push_back (Coordinate (x1 - vCellW/20, y2));
     head.push_back (Coordinate (x1 - vCellW/12, y2 + vCellHt/10));
-    push_back (DrawablePolygon (head));
+    gc.polygon (head);
   }
-  push_back (DrawablePopGraphicContext ());
+
+  gc.pop ();
 }
 
+// ------------------------------------------------------------
+
+static void render_common (gc& gc, const data &d,
+    			   double hscale, double vscale) {
+
+  gc.push ();
+  gc.scaling (hscale, vscale);
+  gc.font (vFont);
+  gc.point_size (vFontPointsize);
+  gc.stroke_width (vLineWidth);
+  gc.stroke_color ("black");
+
+  int labelWidth = label_width (d);
+
+  // draw a "scope-like" diagram for each signal
+  map<signame,int> ypos;
+  int y = 0;
+  for (signal_sequence::const_iterator i = d.sequence.begin ();
+       i != d.sequence.end (); ++ i) {
+    const sigdata &sig = d.find_signal (*i);
+    push_text (gc, vCellWrm, y + vCellHtxt, *i);
+    ypos[*i] = y;
+    int x = labelWidth + vCellWtsep;
+    sigvalue last;
+    for (value_sequence::const_iterator j = sig.data.begin ();
+	 j != sig.data.end (); ++ j) {
+      draw_transition (gc, x, y, last, *j);
+      last = *j;
+      x += vCellW;
+    }
+    y += vCellHt + vCellHdel * sig.maxdelays;
+  }
+
+  // draw the smooth arrows indicating the triggers for signal changes
+  for (list<depdata>::const_iterator i = d.dependencies.begin ();
+       i != d.dependencies.end (); ++ i)
+    draw_dependency (gc, labelWidth + vCellWtsep + vCellWrm + vCellW * i->n_trigger,
+                     vCellHt/2 + ypos[i->trigger],
+		     labelWidth + vCellWtsep + vCellWrm + vCellW * i->n_effect,
+                     vCellHt/2 + ypos[i->effect]);
+
+  // draw the timing delay annotations
+  for (list<delaydata>::const_iterator i = d.delays.begin ();
+       i != d.delays.end (); ++ i)
+    draw_delay (gc, labelWidth + vCellWtsep + vCellWrm + vCellW * i->n_trigger,
+                vCellHt/2 + ypos[i->trigger],
+		labelWidth + vCellWtsep + vCellWrm + vCellW * i->n_effect,
+                vCellHt/2 + ypos[i->effect],
+		ypos[i->trigger] + vCellHt + vCellHdel * i->offset + vCellHtdel,
+                i->text);
+
+  gc.pop ();
+}
+
+// ------------------------------------------------------------
+
+void timing::render (gc &gc, const data &d, double scale) {
+  int base_width, base_height;
+  base_size (d, base_width, base_height);
+
+  gc.width = (int)(scale * base_width);
+  gc.height = (int)(scale * base_height);
+
+  render_common (gc, d, scale, scale);
+}
+
+// ------------------------------------------------------------
+
+void timing::render (gc &gc, const data &d, int w, int h, bool fixAspect) {
+  int base_width, base_height;
+  base_size (d, base_width, base_height);
+
+  gc.width = w;
+  gc.height = h;
+
+  double hscale = w / (double)base_width;
+  double vscale = h / (double)base_height;
+
+  if (fixAspect) {
+      // to maintain aspect ratio, and fit the image:
+      hscale = vscale = min (hscale, vscale);
+  }
+
+  render_common (gc, d, hscale, vscale);
+}
+
+// ------------------------------------------------------------
+
+magick_gc::~magick_gc (void) {
+}
+
+// ------------------------------------------------------------
+
+void magick_gc::bezier (const std::list<Magick::Coordinate> &points) {
+  drawables.push_back (DrawableBezier (points));
+}
+
+// ------------------------------------------------------------
+
+void magick_gc::fill_color (const std::string &name) {
+  drawables.push_back (DrawableFillColor (name));
+}
+
+// ------------------------------------------------------------
+
+void magick_gc::fill_opacity (int op) {
+  drawables.push_back (DrawableFillOpacity (op));
+}
+
+// ------------------------------------------------------------
+
+void magick_gc::font (const std::string& name) {
+  drawables.push_back (DrawableFont (name, AnyStyle, 100, AnyStretch));
+}
+
+// ------------------------------------------------------------
+
+void magick_gc::line (int x1, int y1, int x2, int y2) {
+  drawables.push_back (DrawableLine (x1, y1, x2, y2));
+}
+
+// ------------------------------------------------------------
+
+void magick_gc::point_size (int size) {
+  drawables.push_back (DrawablePointSize (size));
+}
+
+// ------------------------------------------------------------
+
+void magick_gc::polygon (const std::list<Magick::Coordinate> &points)
+{
+  drawables.push_back (DrawablePolygon (points));
+}
+
+// ------------------------------------------------------------
+
+void magick_gc::pop (void)
+{
+  drawables.push_back (DrawablePopGraphicContext ());
+}
+
+// ------------------------------------------------------------
+
+void magick_gc::push (void)
+{
+  drawables.push_back (DrawablePushGraphicContext ());
+}
+
+// ------------------------------------------------------------
+
+void magick_gc::scaling (double hscale, double vscale)
+{
+  drawables.push_back (DrawableScaling (hscale, vscale));
+}
+
+// ------------------------------------------------------------
+
+void magick_gc::stroke_color (const std::string& name)
+{
+  drawables.push_back (DrawableStrokeColor (name));
+}
+
+// ------------------------------------------------------------
+
+void magick_gc::stroke_width (int w)
+{
+  drawables.push_back (DrawableStrokeWidth (w));
+}
+
+// ------------------------------------------------------------
+
+void magick_gc::text (int x, int y, const std::string &text)
+{
+  drawables.push_back (DrawableText (x, y, text));
+}
+
+// ------------------------------------------------------------
+
+void magick_gc::draw (Magick::Image& img) const
+{
+  img.draw (drawables);
+}
+
+// ------------------------------------------------------------
+
+postscript_gc::postscript_gc (void) {
+}
+
+postscript_gc::~postscript_gc (void) {
+}
+
+// ------------------------------------------------------------
+
+void postscript_gc::bezier (const std::list<Magick::Coordinate> &points) {
+  std::list<Magick::Coordinate>::const_iterator i = points.begin();
+
+  ps_text << "newpath\n";
+  ps_text << i->x () << ' ' << (height - i->y ()) << " moveto\n";
+  i++;
+
+  while (i != points.end ()) {
+    ps_text << i->x () << ' ' << (height - i->y ()) << "\n";
+    ++i;
+  }
+
+  ps_text << "curveto\n";
+  ps_text << "stroke\n";
+}
+
+// ------------------------------------------------------------
+
+void postscript_gc::fill_color (const std::string& name) {
+  stroke_color (name);
+}
+
+// ------------------------------------------------------------
+
+void postscript_gc::fill_opacity (int op) {
+}
+
+// ------------------------------------------------------------
+
+void postscript_gc::font (const std::string& name) {
+  ps_text << '/' << name << " findfont\n";
+}
+
+// ------------------------------------------------------------
+
+void postscript_gc::line (int x1, int y1, int x2, int y2) {
+  ps_text << "newpath\n";
+  ps_text << x1 << ' ' << (height - y1) << " moveto\n";
+  ps_text << x2 << ' ' << (height - y2) << " lineto\n";
+  ps_text << "stroke\n";
+}
+
+// ------------------------------------------------------------
+
+void postscript_gc::point_size (int size) {
+  ps_text << size << " scalefont\nsetfont\n";
+}
+
+// ------------------------------------------------------------
+
+void postscript_gc::polygon (const std::list<Magick::Coordinate> &points) {
+  static const char *ops[] = {"stroke", "fill"};
+  std::list<Magick::Coordinate>::const_iterator i;
+  int j;
+
+  for (j = 0; j < 2; j++) {
+    ps_text << "newpath\n";
+    i = points.begin();
+    ps_text << i->x () << ' ' << (height - i->y ()) << " moveto\n";
+    i++;
+
+    while (i != points.end ()) {
+      ps_text << i->x () << ' ' << (height - i->y ()) << " lineto\n";
+      ++i;
+    }
+
+    ps_text << "closepath\n";
+    ps_text << ops[j] << '\n';
+  }
+}
+
+// ------------------------------------------------------------
+
+void postscript_gc::pop (void) {
+  ps_text << "restore\n";
+}
+
+// ------------------------------------------------------------
+
+void postscript_gc::push (void) {
+  ps_text << "save\n";
+}
+
+// ------------------------------------------------------------
+
+void postscript_gc::scaling (double hscale, double vscale) {
+  ps_text << hscale << ' ' << vscale << " scale\n";
+}
+
+// ------------------------------------------------------------
+
+void postscript_gc::stroke_color (const std::string& name) {
+  if (name == "black")
+    ps_text << "0.0 0.0 0.0 setrgbcolor\n";
+  else
+    ps_text << "0.0 0.0 1.0 setrgbcolor\n";
+}
+
+// ------------------------------------------------------------
+
+void postscript_gc::stroke_width (int w) {
+  ps_text << w << " setlinewidth\n";
+}
+
+// ------------------------------------------------------------
+
+void postscript_gc::text (int x, int y, const std::string& text) {
+  unsigned int i;
+
+  ps_text << "newpath\n";
+  ps_text << x << ' ' << (height - y) << " moveto\n";
+
+  ps_text << '(';
+  for (i = 0; i < text.size(); i++) {
+    char c = text[i];
+
+    if (c == '(' || c == ')')
+      ps_text << '\\';
+    ps_text << c;
+  }
+
+  ps_text << ')' << " show\n";
+}
+
+// ------------------------------------------------------------
+
+static std::string filename_ext(const std::string &fname)
+{
+  int i = fname.size () - 1;
+
+  while (i >= 0 && fname[i] != '.')
+    i--;
+
+  return std::string (fname, i + 1);
+}
+
+// ------------------------------------------------------------
+
+void postscript_gc::print (std::ostream& out) const {
+  out << ps_text.str();
+}
+
+// ------------------------------------------------------------
+
+void postscript_gc::print (const std::string& filename) const {
+  std::ofstream out;
+
+  out.exceptions (ofstream::failbit | ofstream::badbit);
+  out.open (filename.c_str());
+
+  std::string ext = filename_ext (filename);
+
+  if (!strcasecmp (ext.c_str (), "eps")) {
+    out << "%PS-Adobe-3.0 EPSF-3.0\n";
+    out << "%%BoundingBox: 0 0 " << width << ' ' << height << '\n';
+    print (out);
+  } else {
+    out << "%PS-Adobe-3.0\n";
+    print (out);
+    out << "showpage\n";
+  }
+
+  out << "%%EOF\n";
+}
+
+// ------------------------------------------------------------
+
+bool postscript_gc::has_ps_ext (const std::string &filename) {
+  std::string ext = filename_ext (filename);
+
+  return !(strcasecmp (ext.c_str (), "ps") &&
+    	   strcasecmp (ext.c_str (), "eps"));
+}
